package com.virnect.tracksample.Renderer;

import android.opengl.GLES32;
import android.util.Log;

/**
 * ShaderManager
 */
public class ShaderManager {

    public int ID; ///< Shader program ID
    private final String TAG = this.getClass().getName(); ///< TAG

    /**
     * Shader Manager constructor
     *
     * @param vertexCode   Vertex code
     * @param fragmentCode Fragment code
     */
    ShaderManager(String vertexCode, String fragmentCode) {
        int vertex, fragment;

        vertex = GLES32.glCreateShader(GLES32.GL_VERTEX_SHADER);
        GLES32.glShaderSource(vertex, vertexCode);
        GLES32.glCompileShader(vertex);
        checkCompileErrors(vertex, "VERTEX");

        fragment = GLES32.glCreateShader(GLES32.GL_FRAGMENT_SHADER);
        GLES32.glShaderSource(fragment, fragmentCode);
        GLES32.glCompileShader(fragment);
        checkCompileErrors(fragment, "FRAGMENT");

        ID = GLES32.glCreateProgram();
        GLES32.glAttachShader(ID, vertex);
        GLES32.glAttachShader(ID, fragment);

        GLES32.glLinkProgram(ID);
        checkCompileErrors(ID, "PROGRAM");

        GLES32.glDeleteShader(vertex);
        GLES32.glDeleteShader(fragment);
    }

    /**
     * Activate this program
     */
    public void use() {
        GLES32.glUseProgram(ID);
    }

    /**
     * Check compile error function
     *
     * @param shader Shader ID
     * @param type   Shader type string
     */
    private void checkCompileErrors(int shader, String type) {
        int[] compiled = new int[1];
        if (!type.equals("PROGRAM")) {
            GLES32.glGetShaderiv(shader, GLES32.GL_COMPILE_STATUS, compiled, 0);
            if (compiled[0] != GLES32.GL_TRUE) {
                String infoLog = GLES32.glGetShaderInfoLog(shader);
                Log.e(TAG, "ERROR::SHADER_COMPILATION_ERROR of type: " + type + "\n " + infoLog);
            }
        } else {
            GLES32.glGetProgramiv(shader, GLES32.GL_LINK_STATUS, compiled, 0);
            if (compiled[0] != GLES32.GL_TRUE) {
                String infoLog = GLES32.glGetProgramInfoLog(shader);
                Log.e(TAG, "ERROR::PROGRAM_LINKING_ERROR of type: " + type + "\n " + infoLog);
            }
        }
    }
}
