package com.virnect.tracksample

class TrackConfig {

    /**
     * Singleton implementation
     */
    companion object {
        private var instance: TrackConfig? = null

        fun instance(): TrackConfig {
            if (instance == null) {
                synchronized(TrackConfig::class.java) {
                    if (instance == null) {
                        instance = TrackConfig()
                    }
                }
            }
            return instance!!
        }
    }

    /**
     * Maximum amount of map targets to be loaded
     */
    val maxAmountOfMapTargets = 3

    /**
     * Indicates which play mode should be initialized
     */
    var playMode = PlayMode.Image

    /**
     * Hybrid target name configuration
     */
    private var targetsImage = arrayOf("VARIMG_000")

    /**
     * Hybrid target name configuration
     */
    private var targetsQR = arrayOf("VARQR_000")

    /**
     * Hybrid target name configuration
     */
    var targetsHybrid = arrayOf("VARIMG_000", "VARQR_000")

    /**
     * Target used to initialize map recording
     */
    val targetMapRecording = arrayOf("VARIMG_000")

    /**
     * Name of the new map target
     * TODO: set by user UI
     */
    var mapRecordingTargetName = "NEW_MAP"

    /**
     * Target used to initialize map recording
     */
    var targetMap: Array<String>? = null

    /**
     * Target used for CAD tracking
     */
    var targetCAD: Array<String>? = null

    /**
     * Target that where chosen through setup
     */
    var targets: Array<String>? = emptyArray()

    /**
     * Definition of target data path for Image and QR-Code targets
     */
    val targetDataPathStandard = "trackAssets/trackTargets/"

    /**
     * Definition of target data path for user recorded map targets
     */
    val targetDataPathMap = "trackAssets/mapTargets/"

    /**
     * Definition of target data path for CAD targets
     */
    val targetDataPathCAD = "trackAssets/trackTargets/cadTargets/"

    /**
     * Definition of used target data path, either targetDataPathStandard or targetDataPathMap
     */
    var targetDataPath = ""

    /**
     * Optional definition of file sequence path
     */
    var fileSequencePath = ""

    /**
     * If the file sequence should be used
     */
    var useFileSequence = false

    /**
     * Determines the content of "targets" dependent on the playMode
     */
    fun setup(mode: PlayMode) {
        playMode = mode

        // Select appropriate target configuration and set global variable
        targets = when (playMode) {
            PlayMode.Image -> targetsImage
            PlayMode.QR -> targetsQR
            PlayMode.Hybrid -> targetsHybrid
            PlayMode.MapRecording -> targetMapRecording
            PlayMode.MapReplay -> targetMap
            PlayMode.CAD -> targetCAD
        }

        // Select appropriate target path
        targetDataPath = when (playMode) {
            PlayMode.MapRecording -> targetDataPathStandard // Location of VARIMG_000 used for initialization
            PlayMode.MapReplay -> targetDataPathMap
            PlayMode.CAD -> targetDataPathCAD
            else -> targetDataPathStandard
        }
    }
}